@@ -12,6 +12,7 @@
 #include <sequoia/openpgp/error.h>
 #include <sequoia/openpgp/crypto.h>
 #include <sequoia/openpgp/packet.h>
+#include <sequoia/openpgp/serialize.h>
 
 /* sequoia::openpgp::KeyID.  */
 
@@ -565,6 +566,32 @@ void pgp_tpk_key_iter_certification_capable (pgp_tpk_key_iter_t iter);
 /*/
 void pgp_tpk_key_iter_signing_capable (pgp_tpk_key_iter_t iter);
 
+/*/
+/// Changes the iterator to only return keys that are capable of
+/// encrypting data at rest.
+///
+/// If you call this function and, e.g., the `signing_capable`
+/// function, the *union* of the values is used.  That is, the
+/// iterator will return keys that are certification capable *or*
+/// signing capable.
+///
+/// Note: you may not call this function after starting to iterate.
+/*/
+void pgp_tpk_key_iter_encrypting_capable_at_rest (pgp_tpk_key_iter_t);
+
+/*/
+/// Changes the iterator to only return keys that are capable of
+/// encrypting data for transport.
+///
+/// If you call this function and, e.g., the `signing_capable`
+/// function, the *union* of the values is used.  That is, the
+/// iterator will return keys that are certification capable *or*
+/// signing capable.
+///
+/// Note: you may not call this function after starting to iterate.
+/*/
+void pgp_tpk_key_iter_encrypting_capable_for_transport (pgp_tpk_key_iter_t);
+
 /*/
 /// Changes the iterator to only return keys that are alive.
 ///
@@ -750,7 +777,7 @@ pgp_tsk_t pgp_tpk_as_tsk (pgp_tpk_t tpk);
 /// The tpk still owns the key.  The caller should neither modify nor
 /// free the key.
 /*/
-pgp_key_t pgp_tpk_primary (pgp_tpk_t tpk);
+pgp_key_t pgp_tpk_primary_key (pgp_tpk_t tpk);
 
 /*/
 /// Returns the TPK's current revocation status.
@@ -1596,10 +1623,10 @@ pgp_writer_stack_t pgp_encryptor_new (pgp_error_t *errp,
 				      pgp_writer_stack_t inner,
 				      char **passwords,
 				      size_t passwords_len,
-				      pgp_tpk_t *recipients,
+				      pgp_recipient_t *recipients,
 				      size_t recipients_len,
-				      pgp_encryption_mode_t mode,
-				      uint8_t cipher_algo);
+				      uint8_t cipher_algo,
+				      uint8_t aead_algo);
 
 /*/
 /// Frees this object.
