@@ -12,6 +12,6 @@ extern crate sequoia_openpgp as openpgp;
 /// Wraps [`sequoia-openpgp::PacketPile`].
 ///
 /// [`sequoia-openpgp::PacketPile`]: ../../sequoia_openpgp/struct.PacketPile.html
-#[::ffi_wrapper_type(prefix = "pgp_",
+#[crate::ffi_wrapper_type(prefix = "pgp_",
                      derive = "Clone, Debug, PartialEq, Parse, Serialize")]
 pub struct PacketPile(openpgp::PacketPile);
