@@ -28,7 +28,7 @@ use self::openpgp::{
     },
 };
 
-use ::error::Status;
+use crate::error::Status;
 use super::fingerprint::Fingerprint;
 use super::packet::key::Key;
 use super::packet::Packet;
@@ -37,11 +37,11 @@ use super::packet_pile::PacketPile;
 use super::tsk::TSK;
 use super::revocation_status::RevocationStatus;
 
-use Maybe;
-use RefRaw;
-use MoveFromRaw;
-use MoveIntoRaw;
-use MoveResultIntoRaw;
+use crate::Maybe;
+use crate::RefRaw;
+use crate::MoveFromRaw;
+use crate::MoveIntoRaw;
+use crate::MoveResultIntoRaw;
 
 /// A transferable public key (TPK).
 ///
@@ -59,7 +59,7 @@ use MoveResultIntoRaw;
 /// passed through as is.
 ///
 /// [RFC 4880, section 11.1]: https://tools.ietf.org/html/rfc4880#section-11.1
-#[::ffi_wrapper_type(
+#[crate::ffi_wrapper_type(
     prefix = "pgp_", name = "tpk",
     derive = "Clone, Debug, Display, PartialEq, Parse, Serialize")]
 pub struct TPK(openpgp::TPK);
@@ -68,7 +68,7 @@ pub struct TPK(openpgp::TPK);
 ///
 /// Consumes `m`.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_from_packet_pile(errp: Option<&mut *mut ::error::Error>,
+fn pgp_tpk_from_packet_pile(errp: Option<&mut *mut crate::error::Error>,
                             m: *mut PacketPile)
                             -> Maybe<TPK> {
     openpgp::TPK::from_packet_pile(m.move_from_raw()).move_into_raw(errp)
@@ -78,7 +78,7 @@ fn pgp_tpk_from_packet_pile(errp: Option<&mut *mut ::error::Error>,
 ///
 /// Consumes the packet parser result.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_from_packet_parser(errp: Option<&mut *mut ::error::Error>,
+fn pgp_tpk_from_packet_parser(errp: Option<&mut *mut crate::error::Error>,
                               ppr: *mut PacketParserResult)
                               -> Maybe<TPK>
 {
@@ -94,7 +94,7 @@ fn pgp_tpk_from_packet_parser(errp: Option<&mut *mut ::error::Error>,
 ///
 /// Consumes `tpk` and `other`.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_merge(errp: Option<&mut *mut ::error::Error>,
+fn pgp_tpk_merge(errp: Option<&mut *mut crate::error::Error>,
                  tpk: *mut TPK,
                  other: *mut TPK)
                  -> Maybe<TPK> {
@@ -111,7 +111,7 @@ fn pgp_tpk_merge(errp: Option<&mut *mut ::error::Error>,
 /// Consumes `tpk` and the packets in `packets`.  The buffer, however,
 /// must be managed by the caller.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_merge_packets(errp: Option<&mut *mut ::error::Error>,
+fn pgp_tpk_merge_packets(errp: Option<&mut *mut crate::error::Error>,
                          tpk: *mut TPK,
                          packets: *mut *mut Packet,
                          packets_len: size_t)
@@ -147,8 +147,10 @@ fn pgp_tpk_as_tsk(tpk: *const TPK) -> *mut TSK<'static> {
 ///
 /// The tpk still owns the key.  The caller must not modify the key.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_primary(tpk: *const TPK) -> *const Key {
-    tpk.ref_raw().primary().move_into_raw()
+fn pgp_tpk_primary_key(tpk: *const TPK) -> *const Key {
+    let key : &self::openpgp::packet::key::UnspecifiedKey
+        = tpk.ref_raw().primary().key().into();
+    key.move_into_raw()
 }
 
 /// Returns the TPK's revocation status as of a given time.
@@ -219,7 +221,7 @@ fn int_to_reason_for_revocation(code: c_int) -> ReasonForRevocation {
 /// assert (revocation);
 /// pgp_signature_free (revocation);    /* Free the generated one.  */
 ///
-/// primary_key = pgp_tpk_primary (tpk);
+/// primary_key = pgp_tpk_primary_key (tpk);
 /// primary_keypair = pgp_key_into_key_pair (NULL, pgp_key_clone (primary_key));
 /// pgp_key_free (primary_key);
 /// assert (primary_keypair);
@@ -242,9 +244,10 @@ fn int_to_reason_for_revocation(code: c_int) -> ReasonForRevocation {
 /// pgp_tpk_free (tpk);
 /// ```
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_revoke(errp: Option<&mut *mut ::error::Error>,
+fn pgp_tpk_revoke(errp: Option<&mut *mut crate::error::Error>,
                   tpk: *const TPK,
-                  primary_signer: *mut Box<crypto::Signer>,
+                  primary_signer: *mut Box<crypto::Signer<
+                          openpgp::packet::key::UnspecifiedRole>>,
                   code: c_int,
                   reason: Option<&c_char>)
                   -> Maybe<Signature>
@@ -286,7 +289,7 @@ fn pgp_tpk_revoke(errp: Option<&mut *mut ::error::Error>,
 /// assert (revocation);
 /// pgp_signature_free (revocation);    /* Free the generated one.  */
 ///
-/// primary_key = pgp_tpk_primary (tpk);
+/// primary_key = pgp_tpk_primary_key (tpk);
 /// primary_keypair = pgp_key_into_key_pair (NULL, pgp_key_clone (primary_key));
 /// pgp_key_free (primary_key);
 /// assert (primary_keypair);
@@ -305,9 +308,10 @@ fn pgp_tpk_revoke(errp: Option<&mut *mut ::error::Error>,
 /// pgp_tpk_free (tpk);
 /// ```
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_revoke_in_place(errp: Option<&mut *mut ::error::Error>,
+fn pgp_tpk_revoke_in_place(errp: Option<&mut *mut crate::error::Error>,
                            tpk: *mut TPK,
-                           primary_signer: *mut Box<crypto::Signer>,
+                           primary_signer: *mut Box<crypto::Signer<
+                                   openpgp::packet::key::UnspecifiedRole>>,
                            code: c_int,
                            reason: Option<&c_char>)
                            -> Maybe<TPK>
@@ -365,8 +369,9 @@ fn pgp_tpk_alive_at(tpk: *const TPK, when: time_t)
 ///
 /// This function consumes `tpk` and returns a new `TPK`.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_set_expiry(errp: Option<&mut *mut ::error::Error>,
-                      tpk: *mut TPK, primary_signer: *mut Box<crypto::Signer>,
+fn pgp_tpk_set_expiry(errp: Option<&mut *mut crate::error::Error>,
+                      tpk: *mut TPK, primary_signer: *mut Box<crypto::Signer<
+                              openpgp::packet::key::UnspecifiedRole>>,
                       expiry: u32)
                       -> Maybe<TPK> {
     let tpk = tpk.move_from_raw();
@@ -459,9 +464,11 @@ pub extern "C" fn pgp_user_id_binding_iter_next<'a>(
 
 /* tpk::KeyIter. */
 
-/// Wrapers a KeyIter for export via the FFI.
+/// Wraps a KeyIter for export via the FFI.
 pub struct KeyIterWrapper<'a> {
-    iter: KeyIter<'a>,
+    pub(crate) // For serialize.rs.
+    iter: KeyIter<'a, openpgp::packet::key::PublicParts,
+                  openpgp::packet::key::UnspecifiedRole>,
     // Whether next has been called.
     next_called: bool,
 }
@@ -562,6 +569,50 @@ pub extern "C" fn pgp_tpk_key_iter_signing_capable<'a>(
     iter_wrapper.iter = tmp.signing_capable();
 }
 
+/// Changes the iterator to only return keys that are capable of
+/// encrypting data at rest.
+///
+/// If you call this function and, e.g., the `signing_capable`
+/// function, the *union* of the values is used.  That is, the
+/// iterator will return keys that are certification capable *or*
+/// signing capable.
+///
+/// Note: you may not call this function after starting to iterate.
+#[::sequoia_ffi_macros::extern_fn] #[no_mangle]
+pub extern "C" fn pgp_tpk_key_iter_encrypting_capable_at_rest<'a>(
+    iter_wrapper: *mut KeyIterWrapper<'a>)
+{
+    let iter_wrapper = ffi_param_ref_mut!(iter_wrapper);
+    if iter_wrapper.next_called {
+        panic!("Can't change KeyIter filter after iterating.");
+    }
+
+    let tmp = std::mem::replace(&mut iter_wrapper.iter, KeyIter::empty());
+    iter_wrapper.iter = tmp.encrypting_capable_at_rest();
+}
+
+/// Changes the iterator to only return keys that are capable of
+/// encrypting data for transport.
+///
+/// If you call this function and, e.g., the `signing_capable`
+/// function, the *union* of the values is used.  That is, the
+/// iterator will return keys that are certification capable *or*
+/// signing capable.
+///
+/// Note: you may not call this function after starting to iterate.
+#[::sequoia_ffi_macros::extern_fn] #[no_mangle]
+pub extern "C" fn pgp_tpk_key_iter_encrypting_capable_for_transport<'a>(
+    iter_wrapper: *mut KeyIterWrapper<'a>)
+{
+    let iter_wrapper = ffi_param_ref_mut!(iter_wrapper);
+    if iter_wrapper.next_called {
+        panic!("Can't change KeyIter filter after iterating.");
+    }
+
+    let tmp = std::mem::replace(&mut iter_wrapper.iter, KeyIter::empty());
+    iter_wrapper.iter = tmp.encrypting_capable_for_transport();
+}
+
 /// Changes the iterator to only return keys that are alive.
 ///
 /// If you call this function (or `pgp_tpk_key_iter_alive_at`), only
@@ -689,13 +740,15 @@ pub extern "C" fn pgp_tpk_key_iter_next<'a>(
             *ptr = rs.move_into_raw();
         }
 
+        let key : &self::openpgp::packet::key::UnspecifiedKey
+            = key.into();
         Some(key).move_into_raw()
     } else {
         None
     }
 }
 
-/// Wrappers a TPKParser for export via the FFI.
+/// Wraps a TPKParser for export via the FFI.
 pub struct TPKParserWrapper<'a> {
     parser: TPKParser<'a, std::vec::IntoIter<self::openpgp::Packet>>,
 }
@@ -705,7 +758,7 @@ pub struct TPKParserWrapper<'a> {
 /// A `TPKParser` parses a keyring, which is simply zero or more TPKs
 /// concatenated together.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_parser_from_bytes(errp: Option<&mut *mut ::error::Error>,
+fn pgp_tpk_parser_from_bytes(errp: Option<&mut *mut crate::error::Error>,
                              buf: *mut u8, len: size_t)
     -> *mut TPKParserWrapper<'static>
 {
@@ -736,7 +789,7 @@ fn pgp_tpk_parser_from_packet_parser(ppr: *mut PacketParserResult<'static>)
 /// If this function returns NULL and does not set *errp, then the end
 /// of the file was reached.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_tpk_parser_next(errp: Option<&mut *mut ::error::Error>,
+fn pgp_tpk_parser_next(errp: Option<&mut *mut crate::error::Error>,
                        parser: *mut TPKParserWrapper)
     -> *mut TPK
 {
@@ -918,7 +971,7 @@ pub extern "C" fn pgp_tpk_builder_add_certification_subkey
 /// Consumes `tpkb`.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle]
 pub extern "C" fn pgp_tpk_builder_generate
-    (errp: Option<&mut *mut ::error::Error>, tpkb: *mut TPKBuilder,
+    (errp: Option<&mut *mut crate::error::Error>, tpkb: *mut TPKBuilder,
      tpk_out: *mut Maybe<TPK>,
      revocation_out: *mut *mut Signature)
     -> Status
