@@ -9,8 +9,8 @@ use std::result;
 
 use quickcheck::{Arbitrary, Gen};
 
-use Error;
-use Result;
+use crate::Error;
+use crate::Result;
 
 /// The OpenPGP public key algorithms as defined in [Section 9.1 of
 /// RFC 4880], and [Section 5 of RFC 6637].
@@ -88,7 +88,7 @@ impl PublicKeyAlgorithm {
 
 impl From<u8> for PublicKeyAlgorithm {
     fn from(u: u8) -> Self {
-        use PublicKeyAlgorithm::*;
+        use crate::PublicKeyAlgorithm::*;
         #[allow(deprecated)]
         match u {
             1 => RSAEncryptSign,
@@ -108,7 +108,7 @@ impl From<u8> for PublicKeyAlgorithm {
 
 impl From<PublicKeyAlgorithm> for u8 {
     fn from(p: PublicKeyAlgorithm) -> u8 {
-        use PublicKeyAlgorithm::*;
+        use crate::PublicKeyAlgorithm::*;
         #[allow(deprecated)]
         match p {
             RSAEncryptSign => 1,
@@ -128,7 +128,7 @@ impl From<PublicKeyAlgorithm> for u8 {
 
 impl fmt::Display for PublicKeyAlgorithm {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        use PublicKeyAlgorithm::*;
+        use crate::PublicKeyAlgorithm::*;
         #[allow(deprecated)]
         match *self {
             RSAEncryptSign => f.write_str("RSA (Encrypt or Sign)"),
@@ -541,9 +541,19 @@ impl Arbitrary for AEADAlgorithm {
 pub enum CompressionAlgorithm {
     /// Null compression.
     Uncompressed,
-    /// DEFLATE
+    /// DEFLATE Compressed Data.
+    ///
+    /// See [RFC 1951] for details.  [Section 9.3 of RFC 4880]
+    /// recommends that this algorithm should be implemented.
+    ///
+    /// [RFC 1951]: https://tools.ietf.org/html/rfc1951
+    /// [Section 9.3 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-9.3
     Zip,
-    /// ZLIB
+    /// ZLIB Compressed Data.
+    ///
+    /// See [RFC 1950] for details.
+    ///
+    /// [RFC 1950]: https://tools.ietf.org/html/rfc1950
     Zlib,
     /// bzip2
     BZip2,
@@ -844,7 +854,7 @@ impl fmt::Display for SignatureType {
             SignatureType::Confirmation =>
                 f.write_str("Confirmation"),
             SignatureType::Unknown(u) =>
-                f.write_fmt(format_args!("Unknown signature type {}", u)),
+                f.write_fmt(format_args!("Unknown signature type 0x{:x}", u)),
         }
     }
 }
@@ -1160,14 +1170,14 @@ mod tests {
 
 
     quickcheck! {
-        fn sigtype_roundtrip(t: SignatureType) -> bool {
+        fn signature_type_roundtrip(t: SignatureType) -> bool {
             let val: u8 = t.clone().into();
             t == SignatureType::from(val)
         }
     }
 
     quickcheck! {
-        fn sigtype_display(t: SignatureType) -> bool {
+        fn signature_type_display(t: SignatureType) -> bool {
             let s = format!("{}", t);
             !s.is_empty()
         }
