@@ -10,13 +10,15 @@ use std::slice;
 use std::vec;
 use std::io;
 
-use Result;
-use Packet;
+use crate::Error;
+use crate::Result;
+use crate::Packet;
 
 pub mod prelude;
 
 pub mod ctb;
 use self::ctb::PacketLengthType;
+use crate::crypto::KeyPair;
 
 use buffered_reader::BufferedReader;
 
@@ -540,8 +542,8 @@ impl<'a> Iterator for PacketPathIter<'a> {
 // Tests the `paths`() iter and `path_ref`().
 #[test]
 fn packet_path_iter() {
-    use parse::Parse;
-    use PacketPile;
+    use crate::parse::Parse;
+    use crate::PacketPile;
 
     fn paths(iter: slice::Iter<Packet>) -> Vec<Vec<usize>> {
         let mut lpaths : Vec<Vec<usize>> = Vec::new();
@@ -562,7 +564,7 @@ fn packet_path_iter() {
 
     for i in 1..5 {
         let pile = PacketPile::from_bytes(
-            ::tests::message(&format!("recursive-{}.gpg", i)[..])).unwrap();
+            crate::tests::message(&format!("recursive-{}.gpg", i)[..])).unwrap();
 
         let mut paths1 : Vec<Vec<usize>> = Vec::new();
         for path in paths(pile.children()).iter() {
@@ -773,12 +775,12 @@ impl From<SKESK> for Packet {
 ///
 ///   [Section 5.5 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.5
 #[derive(PartialEq, Eq, Hash, Clone, Debug)]
-pub enum Key {
+pub enum Key<P: key::KeyParts, R: key::KeyRole> {
     /// Key packet version 4.
-    V4(self::key::Key4),
+    V4(self::key::Key4<P, R>),
 }
 
-impl fmt::Display for Key {
+impl<P: key::KeyParts, R: key::KeyRole> fmt::Display for Key<P, R> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
             Key::V4(k) => k.fmt(f),
@@ -786,7 +788,7 @@ impl fmt::Display for Key {
     }
 }
 
-impl Key {
+impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
     /// Gets the version.
     pub fn version(&self) -> u8 {
         match self {
@@ -805,30 +807,62 @@ impl Key {
             (Key::V4(a), Key::V4(b)) => self::key::Key4::public_cmp(a, b),
         }
     }
+}
 
+impl From<Key<key::PublicParts, key::PrimaryRole>> for Packet {
+    /// Convert the `Key` struct to a `Packet`.
+    fn from(k: Key<key::PublicParts, key::PrimaryRole>) -> Self {
+        Packet::PublicKey(k.into())
+    }
+}
+
+impl From<Key<key::PublicParts, key::SubordinateRole>> for Packet {
+    /// Convert the `Key` struct to a `Packet`.
+    fn from(k: Key<key::PublicParts, key::SubordinateRole>) -> Self {
+        Packet::PublicSubkey(k.into())
+    }
+}
+
+impl From<Key<key::SecretParts, key::PrimaryRole>> for Packet {
+    /// Convert the `Key` struct to a `Packet`.
+    fn from(k: Key<key::SecretParts, key::PrimaryRole>) -> Self {
+        Packet::SecretKey(k.into())
+    }
+}
+
+impl From<Key<key::SecretParts, key::SubordinateRole>> for Packet {
+    /// Convert the `Key` struct to a `Packet`.
+    fn from(k: Key<key::SecretParts, key::SubordinateRole>) -> Self {
+        Packet::SecretSubkey(k.into())
+    }
+}
+
+impl<R: key::KeyRole> Key<key::SecretParts, R> {
     /// Creates a new key pair from a Key packet with an unencrypted
     /// secret key.
     ///
     /// # Errors
     ///
     /// Fails if the secret key is missing, or encrypted.
-    pub fn into_keypair(self) -> Result<::crypto::KeyPair> {
-        match self {
-            Key::V4(p) => p.into_keypair(),
-        }
-    }
+    pub fn into_keypair(mut self) -> Result<KeyPair<R>> {
+        use crate::packet::key::SecretKeyMaterial;
+        let secret = match self.set_secret(None) {
+            Some(SecretKeyMaterial::Unencrypted(secret)) => secret,
+            Some(SecretKeyMaterial::Encrypted(_)) =>
+                return Err(Error::InvalidArgument(
+                    "secret key is encrypted".into()).into()),
+            None =>
+                return Err(Error::InvalidArgument(
+                    "no secret key".into()).into()),
+        };
 
-    /// Convert the `Key` struct to a `Packet`.
-    pub fn into_packet(self, tag: Tag) -> Result<Packet> {
-        match self {
-            Key::V4(p) => p.into_packet(tag),
-        }
+        KeyPair::new(self.into(), secret)
     }
 }
 
 // Trivial forwarder for singleton enum.
-impl Deref for Key {
-    type Target = self::key::Key4;
+impl<P: key::KeyParts, R: key::KeyRole> Deref for Key<P, R> {
+    type Target = self::key::Key4<P, R>;
 
     fn deref(&self) -> &Self::Target {
         match self {
@@ -838,7 +872,7 @@ impl Deref for Key {
 }
 
 // Trivial forwarder for singleton enum.
-impl DerefMut for Key {
+impl<P: key::KeyParts, R: key::KeyRole> DerefMut for Key<P, R> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         match self {
             Key::V4(ref mut p) => p,
@@ -939,3 +973,14 @@ impl DerefMut for AED {
         }
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[test]
+    fn packet_is_send_and_sync() {
+        fn f<T: Send + Sync>(_: T) {}
+        f(Packet::Marker(Default::default()));
+    }
+}
