@@ -2,8 +2,11 @@ use std::fmt;
 use std::str;
 use std::hash::{Hash, Hasher};
 use std::cell::RefCell;
+use std::cmp::Ordering;
+use std::sync::Mutex;
+
 use quickcheck::{Arbitrary, Gen};
-use rfc2822::{
+use crate::rfc2822::{
     AddrSpec,
     AddrSpecOrOther,
     Name,
@@ -12,9 +15,9 @@ use rfc2822::{
 };
 use failure::ResultExt;
 
-use Result;
-use packet;
-use Packet;
+use crate::Result;
+use crate::packet;
+use crate::Packet;
 
 struct ParsedUserID {
     name: Option<String>,
@@ -48,7 +51,7 @@ pub struct UserID {
     /// Use `UserID::default()` to get a UserID with a default settings.
     value: Vec<u8>,
 
-    parsed: RefCell<Option<ParsedUserID>>,
+    parsed: Mutex<RefCell<Option<ParsedUserID>>>,
 }
 
 impl From<Vec<u8>> for UserID {
@@ -56,18 +59,14 @@ impl From<Vec<u8>> for UserID {
         UserID {
             common: Default::default(),
             value: u,
-            parsed: RefCell::new(None),
+            parsed: Mutex::new(RefCell::new(None)),
         }
     }
 }
 
 impl From<&[u8]> for UserID {
     fn from(u: &[u8]) -> Self {
-        UserID {
-            common: Default::default(),
-            value: u.to_vec(),
-            parsed: RefCell::new(None),
-        }
+        u.to_vec().into()
     }
 }
 
@@ -115,14 +114,24 @@ impl fmt::Debug for UserID {
 
 impl PartialEq for UserID {
     fn eq(&self, other: &UserID) -> bool {
-        self.common == other.common
-            && self.value == other.value
+        self.value == other.value
     }
 }
 
 impl Eq for UserID {
 }
 
+impl PartialOrd for UserID {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for UserID {
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.value.cmp(&other.value)
+    }
+}
 
 impl Hash for UserID {
     fn hash<H: Hasher>(&self, state: &mut H) {
@@ -134,11 +143,7 @@ impl Hash for UserID {
 
 impl Clone for UserID {
     fn clone(&self) -> Self {
-        UserID {
-            common: self.common.clone(),
-            value: self.value.clone(),
-            parsed: RefCell::new(None),
-        }
+        self.value.clone().into()
     }
 }
 
@@ -290,10 +295,11 @@ impl UserID {
     }
 
     fn do_parse(&self) -> Result<()> {
-        if self.parsed.borrow().is_none() {
+        if self.parsed.lock().unwrap().borrow().is_none() {
             let s = str::from_utf8(&self.value)?;
 
-            *self.parsed.borrow_mut() = Some(match NameAddrOrOther::parse(s) {
+            *self.parsed.lock().unwrap().borrow_mut() =
+              Some(match NameAddrOrOther::parse(s) {
                 Ok(na) => ParsedUserID {
                     name: na.name().map(|s| s.to_string()),
                     comment: na.comment().map(|s| s.to_string()),
@@ -328,7 +334,7 @@ impl UserID {
     /// input is okay, this still returns the display name.
     pub fn name(&self) -> Result<Option<String>> {
         self.do_parse()?;
-        match *self.parsed.borrow() {
+        match *self.parsed.lock().unwrap().borrow() {
             Some(ParsedUserID { ref name, .. }) =>
                 Ok(name.as_ref().map(|s| s.clone())),
             None => unreachable!(),
@@ -342,7 +348,7 @@ impl UserID {
     /// input is okay, this still returns the first comment.
     pub fn comment(&self) -> Result<Option<String>> {
         self.do_parse()?;
-        match *self.parsed.borrow() {
+        match *self.parsed.lock().unwrap().borrow() {
             Some(ParsedUserID { ref comment, .. }) =>
                 Ok(comment.as_ref().map(|s| s.clone())),
             None => unreachable!(),
@@ -357,7 +363,7 @@ impl UserID {
     /// `UserID::other_address()`.
     pub fn address(&self) -> Result<Option<String>> {
         self.do_parse()?;
-        match *self.parsed.borrow() {
+        match *self.parsed.lock().unwrap().borrow() {
             Some(ParsedUserID { address: Ok(ref address), .. }) =>
                 Ok(Some(address.clone())),
             Some(ParsedUserID { address: Err(_), .. }) =>
@@ -383,7 +389,7 @@ impl UserID {
     /// invalid address can be obtained using `NameAddrOrOther::other()`.
     pub fn other(&self) -> Result<Option<String>> {
         self.do_parse()?;
-        match *self.parsed.borrow() {
+        match *self.parsed.lock().unwrap().borrow() {
             Some(ParsedUserID { ref other, .. }) =>
                 Ok(other.as_ref().map(|s| s.clone())),
             None => unreachable!(),
@@ -403,7 +409,7 @@ impl UserID {
     /// `ssh://server.example.net`.
     pub fn other_or_address(&self) -> Result<Option<String>> {
         self.do_parse()?;
-        match *self.parsed.borrow() {
+        match *self.parsed.lock().unwrap().borrow() {
             Some(ParsedUserID { address: Ok(ref address), .. }) =>
                 Ok(Some(address.clone())),
             Some(ParsedUserID { ref other, .. }) =>
@@ -477,8 +483,8 @@ impl Arbitrary for UserID {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use parse::Parse;
-    use serialize::SerializeInto;
+    use crate::parse::Parse;
+    use crate::serialize::SerializeInto;
 
     quickcheck! {
         fn roundtrip(p: UserID) -> bool {
