@@ -15,7 +15,7 @@ use std::io::{self, Read};
 use std::path::Path;
 
 use buffered_reader::BufferedReader;
-use {
+use crate::{
     Error,
     Fingerprint,
     constants::{
@@ -27,6 +27,7 @@ use {
     packet::{
         BodyLength,
         ctb::CTB,
+        key,
         Key,
         Literal,
         OnePassSig,
@@ -45,7 +46,7 @@ use {
     crypto::SessionKey,
     serialize::Serialize,
 };
-use parse::{
+use crate::parse::{
     Cookie,
     PacketParser,
     PacketParserBuilder,
@@ -146,7 +147,10 @@ pub enum VerificationResult<'a> {
     ///
     /// [web of trust]: https://en.wikipedia.org/wiki/Web_of_trust
     GoodChecksum(Signature,
-                 &'a TPK, &'a Key, Option<&'a Signature>, RevocationStatus<'a>),
+                 &'a TPK,
+                 &'a key::UnspecifiedPublic,
+                 Option<&'a Signature>,
+                 RevocationStatus<'a>),
     /// Unable to verify the signature because the key is missing.
     MissingKey(Signature),
     /// The signature is bad.
@@ -368,7 +372,7 @@ enum IMessageLayer {
 /// Helper for signature verification.
 pub trait VerificationHelper {
     /// Retrieves the TPKs containing the specified keys.
-    fn get_public_keys(&mut self, &[KeyID]) -> Result<Vec<TPK>>;
+    fn get_public_keys(&mut self, _: &[KeyID]) -> Result<Vec<TPK>>;
 
     /// Conveys the message structure.
     ///
@@ -467,6 +471,20 @@ impl<'a, H: VerificationHelper> Verifier<'a, H> {
                                        helper: H, t: time::Tm)
                                        -> Result<Verifier<'a, H>>
     {
+        fn can_sign<P, R>(key: &Key<P, R>, sig: Option<&Signature>, t: time::Tm)
+            -> bool
+            where P: key::KeyParts, R: key::KeyRole
+        {
+            if let Some(sig) = sig {
+                sig.key_flags().can_sign()
+                // Check expiry.
+                    && sig.signature_alive_at(t)
+                    && sig.key_alive_at(key, t)
+            } else {
+                false
+            }
+        }
+
         let mut ppr = PacketParser::from_buffered_reader(bio)?;
 
         let mut v = Verifier {
@@ -499,25 +517,14 @@ impl<'a, H: VerificationHelper> Verifier<'a, H> {
                     v.tpks = v.helper.get_public_keys(&issuers)?;
 
                     for (i, tpk) in v.tpks.iter().enumerate() {
-                        let can_sign = |key: &Key, sig: Option<&Signature>| -> bool {
-                            if let Some(sig) = sig {
-                                sig.key_flags().can_sign()
-                                // Check expiry.
-                                    && sig.signature_alive_at(t)
-                                    && sig.key_alive_at(key, t)
-                            } else {
-                                false
-                            }
-                        };
-
-                        if can_sign(tpk.primary(),
-                                    tpk.primary_key_signature()) {
+                        if can_sign(tpk.primary().key(),
+                                    tpk.primary_key_signature(), t) {
                             v.keys.insert(tpk.keyid(), (i, 0));
                         }
 
                         for (j, skb) in tpk.subkeys().enumerate() {
-                            let key = skb.subkey();
-                            if can_sign(key, skb.binding_signature()) {
+                            let key = skb.key();
+                            if can_sign(key, skb.binding_signature(), t) {
                                 v.keys.insert(key.keyid(),
                                               (i, j + 1));
                             }
@@ -584,7 +591,7 @@ impl<'a, H: VerificationHelper> Verifier<'a, H> {
 
                 // Process the rest of the packets.
                 let mut ppr = PacketParserResult::Some(pp);
-                while let PacketParserResult::Some(mut pp) = ppr {
+                while let PacketParserResult::Some(pp) = ppr {
                     if let Err(err) = pp.possible_message() {
                         return Err(err.context(
                             "Malformed OpenPGP message").into());
@@ -766,7 +773,7 @@ impl<'a> Transformer<'a> {
                 let len = BodyLength::Full((data_prefix.len() + HEADER_LEN) as u32);
                 len.serialize(&mut buf)?;
 
-                let mut lit = Literal::new(DataFormat::Binary);
+                let lit = Literal::new(DataFormat::Binary);
                 lit.serialize_headers(&mut buf, false)?;
 
                 // Copy the data, then proceed directly to the signatures.
@@ -780,7 +787,7 @@ impl<'a> Transformer<'a> {
                 let len = BodyLength::Partial(512);
                 len.serialize(&mut buf)?;
 
-                let mut lit = Literal::new(DataFormat::Binary);
+                let lit = Literal::new(DataFormat::Binary);
                 lit.serialize_headers(&mut buf, false)?;
 
                 // Copy the prefix up to the first chunk, then keep in the
@@ -1284,7 +1291,9 @@ impl<'a, H: VerificationHelper + DecryptionHelper> Decryptor<'a, H> {
                     v.tpks = v.helper.get_public_keys(&issuers)?;
 
                     for (i, tpk) in v.tpks.iter().enumerate() {
-                        let can_sign = |key: &Key, sig: Option<&Signature>| -> bool {
+                        let can_sign = |key: &key::UnspecifiedKey,
+                                        sig: Option<&Signature>| -> bool
+                        {
                             if let Some(sig) = sig {
                                 sig.key_flags().can_sign()
                                 // Check expiry.
@@ -1295,14 +1304,14 @@ impl<'a, H: VerificationHelper + DecryptionHelper> Decryptor<'a, H> {
                             }
                         };
 
-                        if can_sign(tpk.primary(),
+                        if can_sign(tpk.primary().key().into(),
                                     tpk.primary_key_signature()) {
                             v.keys.insert(tpk.keyid(), (i, 0));
                         }
 
                         for (j, skb) in tpk.subkeys().enumerate() {
-                            let key = skb.subkey();
-                            if can_sign(key, skb.binding_signature()) {
+                            let key = skb.key();
+                            if can_sign(key.into(), skb.binding_signature()) {
                                 v.keys.insert(key.keyid(), (i, j + 1));
                             }
                         }
@@ -1377,7 +1386,7 @@ impl<'a, H: VerificationHelper + DecryptionHelper> Decryptor<'a, H> {
                 // Process the rest of the packets.
                 let mut ppr = PacketParserResult::Some(pp);
                 let mut first = true;
-                while let PacketParserResult::Some(mut pp) = ppr {
+                while let PacketParserResult::Some(pp) = ppr {
                     // The literal data packet was already inspected.
                     if first {
                         assert_eq!(pp.packet.tag(), packet::Tag::Literal);
@@ -1455,7 +1464,9 @@ impl<'a, H: VerificationHelper + DecryptionHelper> Decryptor<'a, H> {
                                                     (sig)
                                             } else {
                                                 VerificationResult::GoodChecksum
-                                                    (sig, tpk, key, binding,
+                                                    (sig, tpk,
+                                                     key,
+                                                     binding,
                                                      revocation)
                                             }
                                         } else {
@@ -1541,7 +1552,7 @@ impl<'a, H: VerificationHelper + DecryptionHelper> io::Read for Decryptor<'a, H>
 mod test {
     use failure;
     use super::*;
-    use parse::Parse;
+    use crate::parse::Parse;
 
     #[derive(Debug, PartialEq)]
     struct VHelper {
@@ -1621,7 +1632,7 @@ mod test {
             "neal.pgp",
             "emmelie-dorothea-dina-samantha-awina-ed25519.pgp"
         ].iter()
-         .map(|f| TPK::from_bytes(::tests::key(f)).unwrap())
+         .map(|f| TPK::from_bytes(crate::tests::key(f)).unwrap())
          .collect::<Vec<_>>();
         let tests = &[
             ("messages/signed-1.gpg",                      VHelper::new(1, 0, 0, 0, keys.clone())),
@@ -1630,14 +1641,14 @@ mod test {
             ("keys/neal.pgp",                              VHelper::new(0, 0, 0, 1, keys.clone())),
         ];
 
-        let reference = ::tests::manifesto();
+        let reference = crate::tests::manifesto();
 
         for (f, r) in tests {
             // Test Verifier.
-            let mut h = VHelper::new(0, 0, 0, 0, keys.clone());
+            let h = VHelper::new(0, 0, 0, 0, keys.clone());
             let mut v =
-                match Verifier::from_bytes(::tests::file(f), h,
-                                           ::frozen_time()) {
+                match Verifier::from_bytes(crate::tests::file(f), h,
+                                           crate::frozen_time()) {
                     Ok(v) => v,
                     Err(e) => if r.error > 0 || r.unknown > 0 {
                         // Expected error.  No point in trying to read
@@ -1662,10 +1673,10 @@ mod test {
             assert_eq!(reference, &content[..]);
 
             // Test Decryptor.
-            let mut h = VHelper::new(0, 0, 0, 0, keys.clone());
+            let h = VHelper::new(0, 0, 0, 0, keys.clone());
             let mut v =
-                match Decryptor::from_bytes(::tests::file(f), h,
-                                            ::frozen_time()) {
+                match Decryptor::from_bytes(crate::tests::file(f), h,
+                                            crate::frozen_time()) {
                     Ok(v) => v,
                     Err(e) => if r.error > 0 || r.unknown > 0 {
                         // Expected error.  No point in trying to read
@@ -1738,14 +1749,14 @@ mod test {
 
         // Test verifier.
         let v = Verifier::from_bytes(
-            ::tests::message("signed-1-notarized-by-ed25519.pgp"),
-            VHelper(()), ::frozen_time()).unwrap();
+            crate::tests::message("signed-1-notarized-by-ed25519.pgp"),
+            VHelper(()), crate::frozen_time()).unwrap();
         assert!(v.message_processed());
 
         // Test decryptor.
         let v = Decryptor::from_bytes(
-            ::tests::message("signed-1-notarized-by-ed25519.pgp"),
-            VHelper(()), ::frozen_time()).unwrap();
+            crate::tests::message("signed-1-notarized-by-ed25519.pgp"),
+            VHelper(()), crate::frozen_time()).unwrap();
         assert!(v.message_processed());
     }
 
@@ -1754,16 +1765,16 @@ mod test {
         let keys = [
             "emmelie-dorothea-dina-samantha-awina-ed25519.pgp"
         ].iter()
-         .map(|f| TPK::from_bytes(::tests::key(f)).unwrap())
+         .map(|f| TPK::from_bytes(crate::tests::key(f)).unwrap())
          .collect::<Vec<_>>();
 
-        let reference = ::tests::manifesto();
+        let reference = crate::tests::manifesto();
 
         let h = VHelper::new(0, 0, 0, 0, keys.clone());
         let mut v = DetachedVerifier::from_bytes(
-            ::tests::message("a-cypherpunks-manifesto.txt.ed25519.sig"),
-            ::tests::manifesto(),
-            h, ::frozen_time()).unwrap();
+            crate::tests::message("a-cypherpunks-manifesto.txt.ed25519.sig"),
+            crate::tests::manifesto(),
+            h, crate::frozen_time()).unwrap();
         assert!(v.message_processed());
 
         let mut content = Vec::new();
@@ -1780,9 +1791,9 @@ mod test {
         let h = VHelper::new(0, 0, 0, 0, keys.clone());
         let mut v = DetachedVerifier::from_reader(
             Cursor::new(
-                ::tests::message("a-cypherpunks-manifesto.txt.ed25519.sig")),
-            Cursor::new(::tests::manifesto()),
-            h, ::frozen_time()).unwrap();
+                crate::tests::message("a-cypherpunks-manifesto.txt.ed25519.sig")),
+            Cursor::new(crate::tests::manifesto()),
+            h, crate::frozen_time()).unwrap();
         assert!(v.message_processed());
 
         let mut content = Vec::new();
@@ -1793,9 +1804,9 @@ mod test {
 
     #[test]
     fn verify_long_message() {
-        use constants::DataFormat;
-        use tpk::{TPKBuilder, CipherSuite};
-        use serialize::stream::{LiteralWriter, Signer, Message};
+        use crate::constants::DataFormat;
+        use crate::tpk::{TPKBuilder, CipherSuite};
+        use crate::serialize::stream::{LiteralWriter, Signer, Message};
         use std::io::Write;
 
         let (tpk, _) = TPKBuilder::new()
@@ -1807,7 +1818,7 @@ mod test {
         let mut buf = vec![];
         {
             let key = tpk.keys_all().signing_capable().nth(0).unwrap().2;
-            let mut keypair = key.clone().into_keypair().unwrap();
+            let mut keypair = key.clone().mark_parts_secret().into_keypair().unwrap();
 
             let m = Message::new(&mut buf);
             let signer = Signer::new(m, vec![&mut keypair], None).unwrap();
