@@ -9,43 +9,49 @@
 //!
 //! [encryption example]: struct.Encryptor.html#example
 
+use std::borrow::Borrow;
 use std::fmt;
 use std::io::{self, Write};
-use std::iter;
 use time;
 
-use {
+use crate::{
     crypto,
     Error,
     Fingerprint,
     HashAlgorithm,
+    KeyID,
     Result,
     crypto::Password,
     crypto::SessionKey,
     packet::prelude::*,
     packet::signature,
+    packet::key::{
+        PublicParts,
+        UnspecifiedRole,
+    },
     TPK,
 };
-use packet::ctb::CTB;
-use packet::BodyLength;
+use crate::packet::ctb::CTB;
+use crate::packet::BodyLength;
 use super::{
     PartialBodyFilter,
     Serialize,
     writer,
 };
-use constants::{
+use crate::constants::{
     AEADAlgorithm,
     CompressionAlgorithm,
     DataFormat,
     SignatureType,
     SymmetricAlgorithm,
 };
-use conversions::Time;
+use crate::conversions::Time;
 
 /// Cookie must be public because the writers are.
 #[doc(hidden)]
 #[derive(Debug)]
 pub struct Cookie {
+    pub(crate) // For padding.rs
     level: usize,
     private: Private,
 }
@@ -57,6 +63,7 @@ enum Private {
 }
 
 impl Cookie {
+    pub(crate) // For padding.rs
     fn new(level: usize) -> Self {
         Cookie {
             level: level,
@@ -163,10 +170,10 @@ impl<'a> writer::Stackable<'a, Cookie> for ArbitraryWriter<'a> {
         unreachable!("Only implemented by Signer")
     }
     fn inner_ref(&self) -> Option<&writer::Stackable<'a, Cookie>> {
-        self.inner.inner_ref()
+        Some(self.inner.as_ref())
     }
     fn inner_mut(&mut self) -> Option<&mut writer::Stackable<'a, Cookie>> {
-        self.inner.inner_mut()
+        Some(self.inner.as_mut())
     }
     fn cookie_set(&mut self, cookie: Cookie) -> Cookie {
         self.inner.cookie_set(cookie)
@@ -177,6 +184,9 @@ impl<'a> writer::Stackable<'a, Cookie> for ArbitraryWriter<'a> {
     fn cookie_mut(&mut self) -> &mut Cookie {
         self.inner.cookie_mut()
     }
+    fn position(&self) -> u64 {
+        self.inner.position()
+    }
 }
 
 /// Signs a packet stream.
@@ -186,7 +196,9 @@ impl<'a> writer::Stackable<'a, Cookie> for ArbitraryWriter<'a> {
 /// writes a signature packet.
 ///
 /// Unless otherwise specified, SHA512 is used as hash algorithm.
-pub struct Signer<'a> {
+pub struct Signer<'a, R>
+    where R: key::KeyRole
+{
     // The underlying writer.
     //
     // Because this writer implements `Drop`, we cannot move the inner
@@ -197,14 +209,17 @@ pub struct Signer<'a> {
     // take our inner reader.  If that happens, we only update the
     // digests.
     inner: Option<writer::BoxStack<'a, Cookie>>,
-    signers: Vec<&'a mut dyn crypto::Signer>,
+    signers: Vec<&'a mut dyn crypto::Signer<R>>,
     intended_recipients: Option<Vec<Fingerprint>>,
     detached: bool,
     hash: crypto::hash::Context,
     cookie: Cookie,
+    position: u64,
 }
 
-impl<'a> Signer<'a> {
+impl<'a, R> Signer<'a, R>
+    where R: key::KeyRole
+{
     /// Creates a signer.
     ///
     /// # Example
@@ -215,7 +230,7 @@ impl<'a> Signer<'a> {
     /// use openpgp::constants::DataFormat;
     /// use openpgp::serialize::stream::{Message, Signer, LiteralWriter};
     /// # use openpgp::{Result, TPK};
-    /// # use openpgp::packet::key::SecretKey;
+    /// # use openpgp::packet::prelude::*;
     /// # use openpgp::crypto::KeyPair;
     /// # use openpgp::parse::Parse;
     /// # use openpgp::parse::stream::*;
@@ -223,9 +238,9 @@ impl<'a> Signer<'a> {
     /// #     "../../tests/data/keys/testy-new-private.pgp"))
     /// #     .unwrap();
     /// # let keypair = tsk.keys_valid().signing_capable().nth(0).unwrap().2
-    /// #     .clone().into_keypair().unwrap();
+    /// #     .clone().mark_parts_secret().into_keypair().unwrap();
     /// # f(tsk, keypair).unwrap();
-    /// # fn f(tpk: TPK, mut signing_keypair: KeyPair) -> Result<()> {
+    /// # fn f<R: key::KeyRole>(tpk: TPK, mut signing_keypair: KeyPair<R>) -> Result<()> {
     ///
     /// let mut o = vec![];
     /// {
@@ -265,9 +280,9 @@ impl<'a> Signer<'a> {
     /// # }
     /// ```
     pub fn new<H>(inner: writer::Stack<'a, Cookie>,
-                  signers: Vec<&'a mut dyn crypto::Signer>,
+                  signers: Vec<&'a mut dyn crypto::Signer<R>>,
                   hash_algo: H)
-                  -> Result<writer::Stack<'a, Cookie>>
+        -> Result<writer::Stack<'a, Cookie>>
         where H: Into<Option<HashAlgorithm>>
     {
         Self::make(inner, signers, None, false, hash_algo)
@@ -280,10 +295,10 @@ impl<'a> Signer<'a> {
     /// signature.  This prevents forwarding a signed message using a
     /// different encryption context.
     pub fn with_intended_recipients<H>(inner: writer::Stack<'a, Cookie>,
-                                       signers: Vec<&'a mut dyn crypto::Signer>,
+                                       signers: Vec<&'a mut dyn crypto::Signer<R>>,
                                        recipients: &[&'a TPK],
                                        hash_algo: H)
-                                       -> Result<writer::Stack<'a, Cookie>>
+        -> Result<writer::Stack<'a, Cookie>>
         where H: Into<Option<HashAlgorithm>>
     {
         Self::make(inner, signers,
@@ -300,7 +315,7 @@ impl<'a> Signer<'a> {
     /// use std::io::{Read, Write};
     /// use openpgp::serialize::stream::{Message, Signer, LiteralWriter};
     /// # use openpgp::{Result, TPK};
-    /// # use openpgp::packet::key::SecretKey;
+    /// # use openpgp::packet::prelude::*;
     /// # use openpgp::crypto::KeyPair;
     /// # use openpgp::parse::Parse;
     /// # use openpgp::parse::stream::*;
@@ -308,9 +323,9 @@ impl<'a> Signer<'a> {
     /// #     "../../tests/data/keys/testy-new-private.pgp"))
     /// #     .unwrap();
     /// # let keypair = tsk.keys_valid().signing_capable().nth(0).unwrap().2
-    /// #     .clone().into_keypair().unwrap();
+    /// #     .clone().mark_parts_secret().into_keypair().unwrap();
     /// # f(tsk, keypair).unwrap();
-    /// # fn f(tpk: TPK, mut signing_keypair: KeyPair) -> Result<()> {
+    /// # fn f<R: key::KeyRole>(tpk: TPK, mut signing_keypair: KeyPair<R>) -> Result<()> {
     ///
     /// let mut o = vec![];
     /// {
@@ -353,19 +368,19 @@ impl<'a> Signer<'a> {
     /// # }
     /// ```
     pub fn detached<H>(inner: writer::Stack<'a, Cookie>,
-                       signers: Vec<&'a mut dyn crypto::Signer>,
+                       signers: Vec<&'a mut dyn crypto::Signer<R>>,
                        hash_algo: H)
-                       -> Result<writer::Stack<'a, Cookie>>
+        -> Result<writer::Stack<'a, Cookie>>
         where H: Into<Option<HashAlgorithm>>
     {
         Self::make(inner, signers, None, true, hash_algo)
     }
 
     fn make<H>(inner: writer::Stack<'a, Cookie>,
-               signers: Vec<&'a mut dyn crypto::Signer>,
+               signers: Vec<&'a mut dyn crypto::Signer<R>>,
                intended_recipients: Option<Vec<Fingerprint>>, detached: bool,
                hash_algo: H)
-               -> Result<writer::Stack<'a, Cookie>>
+        -> Result<writer::Stack<'a, Cookie>>
         where H: Into<Option<HashAlgorithm>>
     {
         let mut inner = writer::BoxStack::from(inner);
@@ -401,6 +416,7 @@ impl<'a> Signer<'a> {
                 level: level,
                 private: Private::Signer,
             },
+            position: 0,
         })))
     }
 
@@ -412,7 +428,7 @@ impl<'a> Signer<'a> {
             for signer in self.signers.iter_mut() {
                 // Part of the signature packet is hashed in,
                 // therefore we need to clone the hash.
-                let mut hash = self.hash.clone();
+                let hash = self.hash.clone();
 
                 // Make and hash a signature packet.
                 let mut sig = signature::Builder::new(SignatureType::Binary)
@@ -437,13 +453,17 @@ impl<'a> Signer<'a> {
     }
 }
 
-impl<'a> Drop for Signer<'a> {
+impl<'a, R> Drop for Signer<'a, R>
+    where R: key::KeyRole
+{
     fn drop(&mut self) {
         let _ = self.emit_signatures();
     }
 }
 
-impl<'a> fmt::Debug for Signer<'a> {
+impl<'a, R> fmt::Debug for Signer<'a, R>
+    where R: key::KeyRole
+{
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("Signer")
             .field("inner", &self.inner)
@@ -452,7 +472,9 @@ impl<'a> fmt::Debug for Signer<'a> {
     }
 }
 
-impl<'a> Write for Signer<'a> {
+impl<'a, R> Write for Signer<'a, R>
+    where R: key::KeyRole
+{
     fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
         let written = match self.inner.as_mut() {
             // If we are creating a normal signature, pass data
@@ -468,6 +490,7 @@ impl<'a> Write for Signer<'a> {
 
         if let Ok(amount) = written {
             self.hash.update(&buf[..amount]);
+            self.position += amount as u64;
         }
 
         written
@@ -483,7 +506,9 @@ impl<'a> Write for Signer<'a> {
     }
 }
 
-impl<'a> writer::Stackable<'a, Cookie> for Signer<'a> {
+impl<'a, R> writer::Stackable<'a, Cookie> for Signer<'a, R>
+    where R: key::KeyRole
+{
     fn pop(&mut self) -> Result<Option<writer::BoxStack<'a, Cookie>>> {
         Ok(self.inner.take())
     }
@@ -498,11 +523,7 @@ impl<'a> writer::Stackable<'a, Cookie> for Signer<'a> {
         }
     }
     fn inner_ref(&self) -> Option<&writer::Stackable<'a, Cookie>> {
-        if let Some(ref i) = self.inner {
-            Some(i)
-        } else {
-            None
-        }
+        self.inner.as_ref().map(|r| r.as_ref())
     }
     fn into_inner(mut self: Box<Self>)
                   -> Result<Option<writer::BoxStack<'a, Cookie>>> {
@@ -518,6 +539,9 @@ impl<'a> writer::Stackable<'a, Cookie> for Signer<'a> {
     fn cookie_mut(&mut self) -> &mut Cookie {
         &mut self.cookie
     }
+    fn position(&self) -> u64 {
+        self.position
+    }
 }
 
 
@@ -670,10 +694,10 @@ impl<'a> writer::Stackable<'a, Cookie> for LiteralWriter<'a> {
         unreachable!("Only implemented by Signer")
     }
     fn inner_ref(&self) -> Option<&writer::Stackable<'a, Cookie>> {
-        self.inner.inner_ref()
+        Some(self.inner.as_ref())
     }
     fn inner_mut(&mut self) -> Option<&mut writer::Stackable<'a, Cookie>> {
-        self.inner.inner_mut()
+        Some(self.inner.as_mut())
     }
     fn cookie_set(&mut self, cookie: Cookie) -> Cookie {
         self.inner.cookie_set(cookie)
@@ -684,6 +708,9 @@ impl<'a> writer::Stackable<'a, Cookie> for LiteralWriter<'a> {
     fn cookie_mut(&mut self) -> &mut Cookie {
         self.inner.cookie_mut()
     }
+    fn position(&self) -> u64 {
+        self.inner.position()
+    }
 }
 
 /// Compresses a packet stream.
@@ -707,7 +734,7 @@ impl<'a> writer::Stackable<'a, Cookie> for LiteralWriter<'a> {
 /// {
 ///     let message = Message::new(&mut o);
 ///     let w = Compressor::new(message,
-///                             CompressionAlgorithm::Uncompressed)?;
+///                             CompressionAlgorithm::Uncompressed, None)?;
 ///     let mut w = LiteralWriter::new(w, DataFormat::Text, None, None)?;
 ///     w.write_all(b"Hello world.")?;
 ///     w.finalize()?;
@@ -722,8 +749,14 @@ pub struct Compressor<'a> {
 
 impl<'a> Compressor<'a> {
     /// Creates a new compressor using the given algorithm.
-    pub fn new(inner: writer::Stack<'a, Cookie>, algo: CompressionAlgorithm)
-               -> Result<writer::Stack<'a, Cookie>> {
+    ///
+    /// Passing `None` to `compression_level` selects the default
+    /// compression level.
+    pub fn new<L>(inner: writer::Stack<'a, Cookie>, algo: CompressionAlgorithm,
+                  compression_level: L)
+                  -> Result<writer::Stack<'a, Cookie>>
+        where L: Into<Option<writer::CompressionLevel>>
+    {
         let mut inner = writer::BoxStack::from(inner);
         let level = inner.cookie_ref().level + 1;
 
@@ -733,31 +766,39 @@ impl<'a> Compressor<'a> {
             = PartialBodyFilter::new(writer::Stack::from(inner),
                                      Cookie::new(level));
 
-        Self::new_naked(inner, algo, level)
+        Self::new_naked(inner, algo, compression_level, level)
     }
 
 
     /// Creates a new compressor using the given algorithm.
     pub(crate) // For CompressedData::serialize.
-        fn new_naked(mut inner: writer::Stack<'a, Cookie>, algo: CompressionAlgorithm,
-                     level: usize)
-                 -> Result<writer::Stack<'a, Cookie>> {
+    fn new_naked<L>(mut inner: writer::Stack<'a, Cookie>,
+                    algo: CompressionAlgorithm,
+                    compression_level: L,
+                    level: usize)
+                    -> Result<writer::Stack<'a, Cookie>>
+        where L: Into<Option<writer::CompressionLevel>>
+    {
         // Compressed data header.
         inner.as_mut().write_u8(algo.into())?;
 
         // Create an appropriate filter.
         let inner: writer::Stack<'a, Cookie> = match algo {
-            CompressionAlgorithm::Uncompressed =>
-                writer::Identity::new(inner, Cookie::new(level)),
+            CompressionAlgorithm::Uncompressed => {
+                // Avoid warning about unused value if compiled
+                // without any compression support.
+                let _ = compression_level;
+                writer::Identity::new(inner, Cookie::new(level))
+            },
             #[cfg(feature = "compression-deflate")]
             CompressionAlgorithm::Zip =>
-                writer::ZIP::new(inner, Cookie::new(level)),
+                writer::ZIP::new(inner, Cookie::new(level), compression_level),
             #[cfg(feature = "compression-deflate")]
             CompressionAlgorithm::Zlib =>
-                writer::ZLIB::new(inner, Cookie::new(level)),
+                writer::ZLIB::new(inner, Cookie::new(level), compression_level),
             #[cfg(feature = "compression-bzip2")]
             CompressionAlgorithm::BZip2 =>
-                writer::BZ::new(inner, Cookie::new(level)),
+                writer::BZ::new(inner, Cookie::new(level), compression_level),
             a =>
                 return Err(Error::UnsupportedCompressionAlgorithm(a).into()),
         };
@@ -796,10 +837,10 @@ impl<'a> writer::Stackable<'a, Cookie> for Compressor<'a> {
         unreachable!("Only implemented by Signer")
     }
     fn inner_ref(&self) -> Option<&writer::Stackable<'a, Cookie>> {
-        self.inner.inner_ref()
+        Some(self.inner.as_ref())
     }
     fn inner_mut(&mut self) -> Option<&mut writer::Stackable<'a, Cookie>> {
-        self.inner.inner_mut()
+        Some(self.inner.as_mut())
     }
     fn cookie_set(&mut self, cookie: Cookie) -> Cookie {
         self.inner.cookie_set(cookie)
@@ -810,6 +851,40 @@ impl<'a> writer::Stackable<'a, Cookie> for Compressor<'a> {
     fn cookie_mut(&mut self) -> &mut Cookie {
         self.inner.cookie_mut()
     }
+    fn position(&self) -> u64 {
+        self.inner.position()
+    }
+}
+
+/// A recipient of an encrypted message.
+#[derive(Debug)]
+pub struct Recipient<'a> {
+    keyid: KeyID,
+    key: &'a Key<PublicParts, UnspecifiedRole>,
+}
+
+impl<'a> From<&'a Key<PublicParts, UnspecifiedRole>> for Recipient<'a> {
+    fn from(key: &'a Key<PublicParts, UnspecifiedRole>) -> Self {
+        Self::new(key.keyid(), key)
+    }
+}
+
+impl<'a> Recipient<'a> {
+    /// Creates a new recipient with an explicit recipient keyid.
+    pub fn new(keyid: KeyID, key: &'a Key<PublicParts, UnspecifiedRole>)
+               -> Recipient<'a> {
+        Recipient { keyid, key }
+    }
+
+    /// Gets the KeyID.
+    pub fn keyid(&self) -> &KeyID {
+        &self.keyid
+    }
+
+    /// Sets the KeyID.
+    pub fn set_keyid(&mut self, keyid: KeyID) -> KeyID {
+        std::mem::replace(&mut self.keyid, keyid)
+    }
 }
 
 /// Encrypts a packet stream.
@@ -819,25 +894,6 @@ pub struct Encryptor<'a> {
     cookie: Cookie,
 }
 
-/// Specifies whether to encrypt for archival purposes or for
-/// transport.
-pub enum EncryptionMode {
-    /// Encrypt data for long-term storage.
-    ///
-    /// This should be used for things that should be decryptable for
-    /// a long period of time, e.g. backups, archives, etc.
-    AtRest,
-
-    /// Encrypt data for transport.
-    ///
-    /// This should be used to protect a message in transit.  The
-    /// recipient is expected to take additional steps if she wants to
-    /// be able to decrypt it later on, e.g. store the decrypted
-    /// session key, or re-encrypt the session key with a different
-    /// key.
-    ForTransport,
-}
-
 impl<'a> Encryptor<'a> {
     /// Creates a new encryptor.
     ///
@@ -846,7 +902,10 @@ impl<'a> Encryptor<'a> {
     /// encryption-capable subkeys of the given TPKs.
     ///
     /// Unless otherwise specified, the stream is encrypted using
-    /// AES256.  Key preferences of the recipients are not honored.
+    /// AES256.  If `aead_algo` is `None`, a `SEIP` packet is emitted,
+    /// otherwise the given AEAD algorithm is used.
+    ///
+    /// Key preferences of the recipients are not honored.
     ///
     /// # Example
     ///
@@ -854,8 +913,9 @@ impl<'a> Encryptor<'a> {
     /// use std::io::Write;
     /// extern crate sequoia_openpgp as openpgp;
     /// use openpgp::constants::DataFormat;
+    /// use openpgp::packet::KeyFlags;
     /// use openpgp::serialize::stream::{
-    ///     Message, Encryptor, EncryptionMode, LiteralWriter,
+    ///     Message, Encryptor, LiteralWriter,
     /// };
     /// # use openpgp::Result;
     /// # use openpgp::parse::Parse;
@@ -899,11 +959,20 @@ impl<'a> Encryptor<'a> {
     /// #    */
     /// ).unwrap();
     ///
+    /// // Build a vector of recipients to hand to Encryptor.
+    /// let recipients =
+    ///     tpk.keys_valid()
+    ///     .key_flags(KeyFlags::default()
+    ///                .set_encrypt_at_rest(true)
+    ///                .set_encrypt_for_transport(true))
+    ///     .map(|(_, _, key)| key.into())
+    ///     .collect::<Vec<_>>();
+    ///
     /// let mut o = vec![];
     /// let message = Message::new(&mut o);
     /// let encryptor = Encryptor::new(message,
     ///                                &[&"совершенно секретно".into()],
-    ///                                &[&tpk], EncryptionMode::AtRest, None)
+    ///                                &recipients, None, None)
     ///     .expect("Failed to create encryptor");
     /// let mut w = LiteralWriter::new(encryptor, DataFormat::Text, None, None)?;
     /// w.write_all(b"Hello world.")?;
@@ -911,14 +980,19 @@ impl<'a> Encryptor<'a> {
     /// # Ok(())
     /// # }
     /// ```
-    pub fn new<C>(mut inner: writer::Stack<'a, Cookie>,
-                  passwords: &[&Password], tpks: &[&TPK],
-                  encryption_mode: EncryptionMode,
-                  cipher_algo: C)
-                  -> Result<writer::Stack<'a, Cookie>>
-        where C: Into<Option<SymmetricAlgorithm>>
+    pub fn new<C, A, R>(mut inner: writer::Stack<'a, Cookie>,
+                        passwords: &[&Password],
+                        recipients: R,
+                        cipher_algo: C,
+                        aead_algo: A)
+                        -> Result<writer::Stack<'a, Cookie>>
+        where C: Into<Option<SymmetricAlgorithm>>,
+              A: Into<Option<AEADAlgorithm>>,
+              R: IntoIterator,
+              R::Item: Borrow<Recipient<'a>>
     {
-        if tpks.len() + passwords.len() == 0 {
+        let recipients = recipients.into_iter().collect::<Vec<_>>();
+        if recipients.len() + passwords.len() == 0 {
             return Err(Error::InvalidArgument(
                 "Neither recipient keys nor passwords given".into()).into());
         }
@@ -929,15 +1003,11 @@ impl<'a> Encryptor<'a> {
             nonce: Box<[u8]>,
         }
 
-        // Use AEAD if there are TPKs and all of them support AEAD.
-        let aead = if tpks.len() > 0 && tpks.iter().all(|t| {
-            t.primary_key_signature().map(|s| s.features().supports_aead())
-                .unwrap_or(false)
-        }) {
-            let mut nonce = vec![0; AEADAlgorithm::EAX.iv_size()?];
+        let aead = if let Some(algo) = aead_algo.into() {
+            let mut nonce = vec![0; algo.iv_size()?];
             crypto::random(&mut nonce);
             Some(AEADParameters {
-                algo: AEADAlgorithm::EAX, // Must implement EAX.
+                algo: algo,
                 chunk_size: 4096, // A page, 3 per mille overhead.
                 nonce: nonce.into_boxed_slice(),
             })
@@ -952,58 +1022,11 @@ impl<'a> Encryptor<'a> {
         let sk = SessionKey::new(algo.key_size()?);
 
         // Write the PKESK packet(s).
-        for tpk in tpks {
-            // We need to find all applicable encryption (sub)keys.
-            let can_encrypt = |key: &Key, sig: Option<&Signature>| -> bool {
-                if let Some(sig) = sig {
-                    (match encryption_mode {
-                        EncryptionMode::AtRest =>
-                            sig.key_flags().can_encrypt_at_rest(),
-                        EncryptionMode::ForTransport =>
-                            sig.key_flags().can_encrypt_for_transport(),
-                    }
-                     // Check expiry.
-                     && sig.signature_alive()
-                     && sig.key_alive(key))
-                } else {
-                    false
-                }
-            };
-
-            // Gather all encryption-capable subkeys.
-            let subkeys = tpk.subkeys().filter_map(|skb| {
-                let key = skb.subkey();
-                if can_encrypt(key, skb.binding_signature()) {
-                    Some(key)
-                } else {
-                    None
-                }
-            });
-
-            // Check if the primary key is encryption-capable.
-            let primary_can_encrypt =
-                can_encrypt(tpk.primary(), tpk.primary_key_signature());
-
-            // If the primary key is encryption-capable, prepend to
-            // subkeys via iterator magic.
-            let keys =
-                iter::once(tpk.primary())
-                .filter(|_| primary_can_encrypt)
-                .chain(subkeys);
-
-            let mut count = 0;
-            for key in keys {
-                if let Ok(pkesk) = PKESK3::for_recipient(algo, &sk, key) {
-                    Packet::PKESK(pkesk.into()).serialize(&mut inner)?;
-                    count += 1;
-                }
-            }
-
-            if count == 0 {
-                return Err(Error::InvalidOperation(
-                    format!("Key {} has no suitable encryption subkey",
-                            tpk)).into());
-            }
+        for recipient in recipients {
+            let recipient = recipient.borrow();
+            let mut pkesk = PKESK3::for_recipient(algo, &sk, recipient.key)?;
+            pkesk.set_recipient(recipient.keyid.clone());
+            Packet::PKESK(pkesk.into()).serialize(&mut inner)?;
         }
 
         // Write the SKESK packet(s).
@@ -1084,9 +1107,9 @@ impl<'a> Encryptor<'a> {
 
             // Now recover the original writer.  First, strip the
             // Encryptor.
-            let mut w = w.into_inner()?.unwrap();
+            let w = w.into_inner()?.unwrap();
             // And the partial body filter.
-            let mut w = w.into_inner()?.unwrap();
+            let w = w.into_inner()?.unwrap();
 
             Ok(w)
         } else {
@@ -1139,11 +1162,7 @@ impl<'a> writer::Stackable<'a, Cookie> for Encryptor<'a> {
         unreachable!("Only implemented by Signer")
     }
     fn inner_ref(&self) -> Option<&writer::Stackable<'a, Cookie>> {
-        if let Some(ref i) = self.inner {
-            Some(i)
-        } else {
-            None
-        }
+        self.inner.as_ref().map(|r| r.as_ref())
     }
     fn inner_mut(&mut self) -> Option<&mut writer::Stackable<'a, Cookie>> {
         if let Some(ref mut i) = self.inner {
@@ -1164,15 +1183,18 @@ impl<'a> writer::Stackable<'a, Cookie> for Encryptor<'a> {
     fn cookie_mut(&mut self) -> &mut Cookie {
         &mut self.cookie
     }
+    fn position(&self) -> u64 {
+        self.inner.as_ref().map(|i| i.position()).unwrap_or(0)
+    }
 }
 
 #[cfg(test)]
 mod test {
     use std::io::Read;
-    use {Packet, PacketPile, packet::CompressedData};
-    use parse::{Parse, PacketParserResult, PacketParser};
+    use crate::{Packet, PacketPile, packet::CompressedData};
+    use crate::parse::{Parse, PacketParserResult, PacketParser};
     use super::*;
-    use constants::DataFormat::Text as T;
+    use crate::constants::DataFormat::Text as T;
 
     #[test]
     fn arbitrary() {
@@ -1230,7 +1252,7 @@ mod test {
         {
             let m = Message::new(&mut o);
             let c = Compressor::new(
-                m, CompressionAlgorithm::Uncompressed).unwrap();
+                m, CompressionAlgorithm::Uncompressed, None).unwrap();
             let mut ls = LiteralWriter::new(c, T, None, None).unwrap();
             write!(ls, "one").unwrap();
             let c = ls.finalize_one().unwrap().unwrap(); // Pop the LiteralWriter.
@@ -1289,9 +1311,9 @@ mod test {
         {
             let m = Message::new(&mut o);
             let c0 = Compressor::new(
-                m, CompressionAlgorithm::Uncompressed).unwrap();
+                m, CompressionAlgorithm::Uncompressed, None).unwrap();
             let c = Compressor::new(
-                c0, CompressionAlgorithm::Uncompressed).unwrap();
+                c0, CompressionAlgorithm::Uncompressed, None).unwrap();
             let mut ls = LiteralWriter::new(c, T, None, None).unwrap();
             write!(ls, "one").unwrap();
             let c = ls.finalize_one().unwrap().unwrap();
@@ -1300,7 +1322,7 @@ mod test {
             let c = ls.finalize_one().unwrap().unwrap();
             let c0 = c.finalize_one().unwrap().unwrap();
             let c = Compressor::new(
-                c0, CompressionAlgorithm::Uncompressed).unwrap();
+                c0, CompressionAlgorithm::Uncompressed, None).unwrap();
             let mut ls = LiteralWriter::new(c, T, None, None).unwrap();
             write!(ls, "three").unwrap();
             let c = ls.finalize_one().unwrap().unwrap();
@@ -1319,7 +1341,7 @@ mod test {
         }
     }
 
-    #[cfg(feature = "compression-deflate")]
+    #[cfg(feature = "compression-bzip2")]
     #[test]
     fn stream_big() {
         let zeros = vec![0; 1024 * 1024 * 4];
@@ -1327,7 +1349,7 @@ mod test {
         {
             let m = Message::new(&mut o);
             let c = Compressor::new(m,
-                                    CompressionAlgorithm::BZip2).unwrap();
+                                    CompressionAlgorithm::BZip2, None).unwrap();
             let mut ls = LiteralWriter::new(c, T, None, None).unwrap();
             // Write 64 megabytes of zeroes.
             for _ in 0 .. 16 {
@@ -1339,14 +1361,14 @@ mod test {
 
     #[test]
     fn signature() {
-        use crypto::KeyPair;
+        use crate::crypto::KeyPair;
         use std::collections::HashMap;
-        use Fingerprint;
+        use crate::Fingerprint;
 
-        let mut keys: HashMap<Fingerprint, Key> = HashMap::new();
+        let mut keys: HashMap<Fingerprint, key::UnspecifiedPublic> = HashMap::new();
         for tsk in &[
-            TPK::from_bytes(::tests::key("testy-private.pgp")).unwrap(),
-            TPK::from_bytes(::tests::key("testy-new-private.pgp")).unwrap(),
+            TPK::from_bytes(crate::tests::key("testy-private.pgp")).unwrap(),
+            TPK::from_bytes(crate::tests::key("testy-new-private.pgp")).unwrap(),
         ] {
             for key in tsk.keys_all().signing_capable().map(|x| x.2)
             {
@@ -1357,15 +1379,15 @@ mod test {
         let mut o = vec![];
         {
             let mut signers = keys.iter().map(|(_, key)| {
-                key.clone().into_keypair()
+                key.clone().mark_parts_secret().into_keypair()
                     .expect("expected unencrypted secret key")
-            }).collect::<Vec<KeyPair>>();
+            }).collect::<Vec<KeyPair<_>>>();
 
             let m = Message::new(&mut o);
             let signer = Signer::new(
                 m,
                 signers.iter_mut()
-                    .map(|s| -> &mut dyn crypto::Signer {s})
+                    .map(|s| -> &mut dyn crypto::Signer<_> {s})
                     .collect(),
                 None)
                 .unwrap();
@@ -1404,7 +1426,7 @@ mod test {
             let m = Message::new(&mut o);
             let encryptor = Encryptor::new(
                 m, &passwords.iter().collect::<Vec<&Password>>(),
-                &[], EncryptionMode::ForTransport, None)
+                &[], None, None)
                 .unwrap();
             let mut literal = LiteralWriter::new(encryptor, DataFormat::Binary,
                                                  None, None)
