@@ -1,18 +1,21 @@
 use time;
 
-use packet;
-use packet::{Features, KeyFlags};
-use packet::Key;
-use packet::key::Key4;
-use Result;
-use packet::Signature;
-use packet::signature;
-use TPK;
-use Error;
-use conversions::Time;
-use crypto::Password;
-use autocrypt::Autocrypt;
-use constants::{
+use crate::packet;
+use crate::packet::{Features, KeyFlags};
+use crate::packet::{
+    key,
+    Key,
+    key::Key4,
+};
+use crate::Result;
+use crate::packet::Signature;
+use crate::packet::signature;
+use crate::TPK;
+use crate::Error;
+use crate::conversions::Time;
+use crate::crypto::Password;
+use crate::autocrypt::Autocrypt;
+use crate::constants::{
     HashAlgorithm,
     SignatureType,
     SymmetricAlgorithm,
@@ -42,8 +45,11 @@ impl Default for CipherSuite {
 }
 
 impl CipherSuite {
-    fn generate_key(self, flags: &KeyFlags) -> Result<Key> {
-        use constants::Curve;
+    fn generate_key<R>(self, flags: &KeyFlags)
+        -> Result<Key<key::SecretParts, R>>
+        where R: key::KeyRole
+    {
+        use crate::constants::Curve;
 
         match self {
             CipherSuite::RSA2k =>
@@ -272,8 +278,8 @@ impl TPKBuilder {
 
     /// Generates the actual TPK.
     pub fn generate(mut self) -> Result<(TPK, Signature)> {
-        use {PacketPile, Packet};
-        use constants::ReasonForRevocation;
+        use crate::{PacketPile, Packet};
+        use crate::constants::ReasonForRevocation;
 
         let mut packets = Vec::<Packet>::with_capacity(
             1 + 1 + self.subkeys.len() + self.userids.len()
@@ -286,7 +292,7 @@ impl TPKBuilder {
 
         // Generate & and self-sign primary key.
         let (primary, sig) = self.primary_key()?;
-        let mut signer = primary.clone().into_keypair().unwrap();
+        let mut signer = primary.clone().mark_parts_secret().into_keypair().unwrap();
 
         packets.push(Packet::PublicKey({
             let mut primary = primary.clone();
@@ -303,7 +309,7 @@ impl TPKBuilder {
         // Sign UserIDs.
         for uid in self.userids.into_iter() {
             let builder = signature::Builder::from(sig.clone())
-                .set_sigtype(SignatureType::PositiveCertificate);
+                .set_type(SignatureType::PositiveCertificate);
             let signature = uid.bind(&mut signer, &tpk, builder, None, None)?;
             tpk = tpk.merge_packets(vec![uid.into(), signature.into()])?;
         }
@@ -311,7 +317,7 @@ impl TPKBuilder {
         // Sign UserAttributes.
         for ua in self.user_attributes.into_iter() {
             let builder = signature::Builder::from(sig.clone())
-                .set_sigtype(SignatureType::PositiveCertificate);
+                .set_type(SignatureType::PositiveCertificate);
             let signature = ua.bind(&mut signer, &tpk, builder, None, None)?;
             tpk = tpk.merge_packets(vec![ua.into(), signature.into()])?;
         }
@@ -321,10 +327,6 @@ impl TPKBuilder {
             let flags = &blueprint.flags;
             let mut subkey = self.ciphersuite.generate_key(flags)?;
 
-            if let Some(ref password) = self.password {
-                subkey.secret_mut().unwrap().encrypt_in_place(password)?;
-            }
-
             let mut builder =
                 signature::Builder::new(SignatureType::SubkeyBinding)
                 .set_features(&Features::sequoia())?
@@ -350,13 +352,18 @@ impl TPKBuilder {
                     .set_signature_creation_time(time::now().canonicalize())?
                     .set_issuer_fingerprint(subkey.fingerprint())?
                     .set_issuer(subkey.keyid())?
-                    .sign_subkey_binding(&mut subkey_signer, &primary, &subkey,
+                    .sign_subkey_binding(&mut subkey_signer, &primary,
+                                         &subkey.mark_parts_public_ref(),
                                          HashAlgorithm::SHA512)?;
                 builder = builder.set_embedded_signature(backsig)?;
             }
 
-            let signature =
-                subkey.bind(&mut signer, &tpk, builder, None, None)?;
+            let signature = subkey.mark_parts_public_ref()
+                .bind(&mut signer, &tpk, builder, None, None)?;
+
+            if let Some(ref password) = self.password {
+                subkey.secret_mut().unwrap().encrypt_in_place(password)?;
+            }
             tpk = tpk.merge_packets(vec![Packet::SecretSubkey(subkey),
                                          signature.into()])?;
         }
@@ -373,10 +380,8 @@ impl TPKBuilder {
     }
 
     fn primary_key(&self)
-        -> Result<(Key, Signature)>
+        -> Result<(key::PublicKey, Signature)>
     {
-        use SignatureType;
-
         let key = self.ciphersuite.generate_key(
             &KeyFlags::default().set_certify(true))?;
         let sig = signature::Builder::new(SignatureType::DirectKey)
@@ -393,15 +398,15 @@ impl TPKBuilder {
         let sig = sig.sign_primary_key_binding(&mut signer,
                                                HashAlgorithm::SHA512)?;
 
-        Ok((key, sig.into()))
+        Ok((key.mark_parts_public(), sig.into()))
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use packet::signature::subpacket::{SubpacketTag, Subpacket, SubpacketValue};
-    use constants::PublicKeyAlgorithm;
+    use crate::packet::signature::subpacket::{SubpacketTag, Subpacket, SubpacketValue};
+    use crate::constants::PublicKeyAlgorithm;
 
     #[test]
     fn all_opts() {
@@ -415,7 +420,7 @@ mod tests {
             .generate().unwrap();
 
         let mut userids = tpk.userids()
-            .map(|u| String::from_utf8_lossy(u.userid.value()).into_owned())
+            .map(|u| String::from_utf8_lossy(u.userid().value()).into_owned())
             .collect::<Vec<String>>();
         userids.sort();
 
@@ -436,8 +441,8 @@ mod tests {
             .generate().unwrap();
 
         assert_eq!(tpk.userids().count(), 0);
-        assert_eq!(tpk.primary_key_signature().unwrap().sigtype(),
-                   ::constants::SignatureType::DirectKey);
+        assert_eq!(tpk.primary_key_signature().unwrap().typ(),
+                   crate::constants::SignatureType::DirectKey);
         assert_eq!(tpk.subkeys().count(), 3);
         if let Some(sig) = tpk.primary_key_signature() {
             assert!(sig.features().supports_mdc());
@@ -454,16 +459,16 @@ mod tests {
             .set_cipher_suite(CipherSuite::RSA3k)
             .set_cipher_suite(CipherSuite::Cv25519)
             .generate().unwrap();
-        assert_eq!(tpk1.primary().pk_algo(), PublicKeyAlgorithm::EdDSA);
+        assert_eq!(tpk1.primary().key().pk_algo(), PublicKeyAlgorithm::EdDSA);
 
         let (tpk2, _) = TPKBuilder::new()
             .set_cipher_suite(CipherSuite::RSA3k)
             .add_userid("test2@example.com")
             .add_encryption_subkey()
             .generate().unwrap();
-        assert_eq!(tpk2.primary().pk_algo(),
+        assert_eq!(tpk2.primary().key().pk_algo(),
                    PublicKeyAlgorithm::RSAEncryptSign);
-        assert_eq!(tpk2.subkeys().next().unwrap().subkey().pk_algo(),
+        assert_eq!(tpk2.subkeys().next().unwrap().key().pk_algo(),
                    PublicKeyAlgorithm::RSAEncryptSign);
     }
 
@@ -472,7 +477,7 @@ mod tests {
         let (tpk1, _) = TPKBuilder::new()
             .add_userid("test2@example.com")
             .generate().unwrap();
-        assert_eq!(tpk1.primary().pk_algo(),
+        assert_eq!(tpk1.primary().key().pk_algo(),
                    PublicKeyAlgorithm::EdDSA);
         assert!(tpk1.subkeys().next().is_none());
         if let Some(sig) = tpk1.primary_key_signature() {
@@ -488,9 +493,9 @@ mod tests {
         let (tpk1, _) = TPKBuilder::autocrypt(Autocrypt::V1,
                                               Some("Foo"))
             .generate().unwrap();
-        assert_eq!(tpk1.primary().pk_algo(),
+        assert_eq!(tpk1.primary().key().pk_algo(),
                    PublicKeyAlgorithm::RSAEncryptSign);
-        assert_eq!(tpk1.subkeys().next().unwrap().subkey().pk_algo(),
+        assert_eq!(tpk1.subkeys().next().unwrap().key().pk_algo(),
                    PublicKeyAlgorithm::RSAEncryptSign);
         assert_eq!(tpk1.userids().count(), 1);
     }
@@ -500,14 +505,14 @@ mod tests {
         let (tpk1, _) = TPKBuilder::autocrypt(Autocrypt::V1_1,
                                               Some("Foo"))
             .generate().unwrap();
-        assert_eq!(tpk1.primary().pk_algo(),
+        assert_eq!(tpk1.primary().key().pk_algo(),
                    PublicKeyAlgorithm::EdDSA);
-        assert_eq!(tpk1.subkeys().next().unwrap().subkey().pk_algo(),
+        assert_eq!(tpk1.subkeys().next().unwrap().key().pk_algo(),
                    PublicKeyAlgorithm::ECDH);
         assert_match!(
-            ::crypto::mpis::PublicKey::ECDH {
-                curve: ::constants::Curve::Cv25519, ..
-            } = tpk1.subkeys().next().unwrap().subkey().mpis());
+            crate::crypto::mpis::PublicKey::ECDH {
+                curve: crate::constants::Curve::Cv25519, ..
+            } = tpk1.subkeys().next().unwrap().key().mpis());
         assert_eq!(tpk1.userids().count(), 1);
     }
 
@@ -551,7 +556,7 @@ mod tests {
 
     #[test]
     fn generate_revocation_certificate() {
-        use RevocationStatus;
+        use crate::RevocationStatus;
         let (tpk, revocation) = TPKBuilder::new()
             .set_cipher_suite(CipherSuite::Cv25519)
             .generate().unwrap();
@@ -565,7 +570,7 @@ mod tests {
 
     #[test]
     fn builder_roundtrip() {
-        use PacketPile;
+        use crate::PacketPile;
 
         let (tpk,_) = TPKBuilder::new()
             .set_cipher_suite(CipherSuite::Cv25519)
@@ -584,7 +589,7 @@ mod tests {
             .set_cipher_suite(CipherSuite::Cv25519)
             .set_password(Some(String::from("streng geheim").into()))
             .generate().unwrap();
-        assert!(tpk.primary().secret().unwrap().is_encrypted());
+        assert!(tpk.primary().key().secret().unwrap().is_encrypted());
     }
 
     #[test]
