@@ -7,8 +7,10 @@ extern crate failure;
 extern crate prettytable;
 extern crate rpassword;
 extern crate tempfile;
+extern crate termsize;
 extern crate time;
 extern crate itertools;
+extern crate tokio_core;
 
 use failure::ResultExt;
 use prettytable::{Table, Cell, Row};
@@ -22,11 +24,11 @@ extern crate sequoia_core;
 extern crate sequoia_net;
 extern crate sequoia_store;
 
-use openpgp::{armor, autocrypt, Fingerprint, TPK};
-use openpgp::conversions::hex;
-use openpgp::parse::Parse;
-use openpgp::serialize::Serialize;
-use openpgp::tpk::armor::Encoder;
+use crate::openpgp::{armor, autocrypt, Fingerprint, TPK};
+use crate::openpgp::conversions::hex;
+use crate::openpgp::packet::KeyFlags;
+use crate::openpgp::parse::Parse;
+use crate::openpgp::serialize::Serialize;
 use sequoia_core::{Context, NetworkPolicy};
 use sequoia_net::{KeyServer, wkd};
 use sequoia_store::{Store, LogIter};
@@ -75,6 +77,52 @@ fn load_tpks<'a, I>(files: I) -> openpgp::Result<Vec<TPK>>
     Ok(tpks)
 }
 
+/// Serializes a keyring, adding descriptive headers if armored.
+fn serialize_keyring(mut output: &mut io::Write, tpks: &[TPK], binary: bool)
+                     -> openpgp::Result<()> {
+    // Handle the easy options first.  No armor no cry:
+    if binary {
+        for tpk in tpks {
+            tpk.serialize(&mut output)?;
+        }
+        return Ok(());
+    }
+
+    // Just one TPK?  Ez:
+    if tpks.len() == 1 {
+        return tpks[0].armored().serialize(&mut output);
+    }
+
+    // Otherwise, collect the headers first:
+    let mut headers = Vec::new();
+    for (i, tpk) in tpks.iter().enumerate() {
+        headers.push(format!("Key #{}", i));
+        headers.append(&mut tpk.armor_headers());
+    }
+
+    let headers: Vec<_> = headers.iter()
+        .map(|value| ("Comment", value.as_str()))
+        .collect();
+    let mut output = armor::Writer::new(&mut output,
+                                        armor::Kind::PublicKey,
+                                        &headers)?;
+    for tpk in tpks {
+        tpk.serialize(&mut output)?;
+    }
+    Ok(())
+}
+
+fn parse_armor_kind(kind: Option<&str>) -> armor::Kind {
+    match kind.expect("has default value") {
+        "message" => armor::Kind::Message,
+        "publickey" => armor::Kind::PublicKey,
+        "secretkey" => armor::Kind::SecretKey,
+        "signature" => armor::Kind::Signature,
+        "file" => armor::Kind::File,
+        _ => unreachable!(),
+    }
+}
+
 /// Prints a warning if the user supplied "help" or "-help" to an
 /// positional argument.
 ///
@@ -116,6 +164,7 @@ fn real_main() -> Result<(), failure::Error> {
         builder = builder.home(dir);
     }
     let ctx = builder.build()?;
+    let mut core = tokio_core::reactor::Core::new()?;
 
     match matches.subcommand() {
         ("decrypt",  Some(m)) => {
@@ -158,9 +207,21 @@ fn real_main() -> Result<(), failure::Error> {
             let additional_secrets = m.values_of("signer-key-file")
                 .map(load_tpks)
                 .unwrap_or(Ok(vec![]))?;
+            let mode = match m.value_of("mode").expect("has default") {
+                "rest" => KeyFlags::default()
+                    .set_encrypt_at_rest(true),
+                "transport" => KeyFlags::default()
+                    .set_encrypt_for_transport(true),
+                "all" => KeyFlags::default()
+                    .set_encrypt_at_rest(true)
+                    .set_encrypt_for_transport(true),
+                _ => unreachable!("uses possible_values"),
+            };
             commands::encrypt(&mut store, &mut input, &mut output,
                               m.occurrences_of("symmetric") as usize,
-                              recipients, additional_tpks, additional_secrets)?;
+                              recipients, additional_tpks, additional_secrets,
+                              mode,
+                              m.value_of("compression").expect("has default"))?;
         },
         ("sign",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
@@ -198,14 +259,7 @@ fn real_main() -> Result<(), failure::Error> {
         ("enarmor",  Some(m)) => {
             let mut input = open_or_stdin(m.value_of("input"))?;
             let mut output = create_or_stdout(m.value_of("output"), force)?;
-            let kind = match m.value_of("kind").expect("has default value") {
-                "message" => armor::Kind::Message,
-                "publickey" => armor::Kind::PublicKey,
-                "secretkey" => armor::Kind::SecretKey,
-                "signature" => armor::Kind::Signature,
-                "file" => armor::Kind::File,
-                _ => unreachable!(),
-            };
+            let kind = parse_armor_kind(m.value_of("kind"));
             let mut filter = armor::Writer::new(&mut output, kind, &[])?;
             io::copy(&mut input, &mut filter)?;
         },
@@ -218,7 +272,7 @@ fn real_main() -> Result<(), failure::Error> {
         ("autocrypt", Some(m)) => {
             match m.subcommand() {
                 ("decode",  Some(m)) => {
-                    let mut input = open_or_stdin(m.value_of("input"))?;
+                    let input = open_or_stdin(m.value_of("input"))?;
                     let mut output = create_or_stdout(m.value_of("output"), force)?;
                     let ac = autocrypt::AutocryptHeaders::from_reader(input)?;
                     for h in &ac.headers {
@@ -230,7 +284,7 @@ fn real_main() -> Result<(), failure::Error> {
                     }
                 },
                 ("encode-sender",  Some(m)) => {
-                    let mut input = open_or_stdin(m.value_of("input"))?;
+                    let input = open_or_stdin(m.value_of("input"))?;
                     let mut output = create_or_stdout(m.value_of("output"),
                                                       force)?;
                     let tpk = TPK::from_reader(input)?;
@@ -272,9 +326,10 @@ fn real_main() -> Result<(), failure::Error> {
                     } else {
                         None
                     };
+                let width = termsize::get().map(|s| s.cols as usize);
                 commands::dump(&mut input, &mut output,
                                m.is_present("mpis"), m.is_present("hex"),
-                               session_key.as_ref())?;
+                               session_key.as_ref(), width)?;
             },
             ("split",  Some(m)) => {
                 let mut input = open_or_stdin(m.value_of("input"))?;
@@ -294,6 +349,16 @@ fn real_main() -> Result<(), failure::Error> {
                             + "-");
                 commands::split(&mut input, &prefix)?;
             },
+            ("join",  Some(m)) => {
+                let output = create_or_stdout(m.value_of("output"), force)?;
+                let mut output = if ! m.is_present("binary") {
+                    let kind = parse_armor_kind(m.value_of("kind"));
+                    Box::new(armor::Writer::new(output, kind, &[])?)
+                } else {
+                    output
+                };
+                commands::join(m.values_of("input"), &mut output)?;
+            },
             _ => unreachable!(),
         },
 
@@ -301,7 +366,7 @@ fn real_main() -> Result<(), failure::Error> {
             let mut ks = if let Some(uri) = m.value_of("server") {
                 KeyServer::new(&ctx, &uri)
             } else {
-                KeyServer::sks_pool(&ctx)
+                KeyServer::keys_openpgp_org(&ctx)
             }.context("Malformed keyserver URI")?;
 
             match m.subcommand() {
@@ -317,25 +382,20 @@ fn real_main() -> Result<(), failure::Error> {
                     let id = id.unwrap();
 
                     let mut output = create_or_stdout(m.value_of("output"), force)?;
-                    let mut output = if ! m.is_present("binary") {
-                        Box::new(armor::Writer::new(&mut output,
-                                                    armor::Kind::PublicKey,
-                                                    &[])?)
+                    let tpk = core.run(ks.get(&id))
+                        .context("Failed to retrieve key")?;
+                    if ! m.is_present("binary") {
+                        tpk.armored().serialize(&mut output)
                     } else {
-                        output
-                    };
-
-                    ks.get(&id)
-                        .context("Failed to retrieve key")?
-                    .serialize(&mut output)
-                        .context("Failed to serialize key")?;
+                        tpk.serialize(&mut output)
+                    }.context("Failed to serialize key")?;
                 },
                 ("send",  Some(m)) => {
                     let mut input = open_or_stdin(m.value_of("input"))?;
                     let tpk = TPK::from_reader(&mut input).
                         context("Malformed key")?;
 
-                    ks.send(&tpk)
+                    core.run(ks.send(&tpk))
                         .context("Failed to send key to server")?;
                 },
                 _ => unreachable!(),
@@ -367,7 +427,7 @@ fn real_main() -> Result<(), failure::Error> {
                     if m.is_present("binary") {
                         tpk.serialize(&mut output)?;
                     } else {
-                        Encoder::new(&tpk).serialize(&mut output)?;
+                        tpk.armored().serialize(&mut output)?;
                     }
                 },
                 ("delete",  Some(m)) => {
@@ -472,7 +532,7 @@ fn real_main() -> Result<(), failure::Error> {
                     // stderr and exit.
                     // Because it might be created a WkdServer struct, not
                     // doing it for now.
-                    let tpks = wkd::get(&email_address)?;
+                    let tpks = core.run(wkd::get(&email_address))?;
                     // ```text
                     //     The HTTP GET method MUST return the binary representation of the
                     //     OpenPGP key for the given mail address.
@@ -481,17 +541,8 @@ fn real_main() -> Result<(), failure::Error> {
                     // But to keep the parallelism with `store export` and `keyserver get`,
                     // The output is armored if not `--binary` option is given.
                     let mut output = create_or_stdout(m.value_of("output"), force)?;
-                    let mut output = if ! m.is_present("binary") {
-                        Box::new(armor::Writer::new(&mut output,
-                                                    armor::Kind::PublicKey,
-                                                    &[])?)
-                    } else {
-                        output
-                    };
-
-                    for tpk in tpks {
-                        tpk.serialize(&mut output)?;
-                    }
+                    serialize_keyring(&mut output, &tpks,
+                                      m.is_present("binary"))?;
                 },
                 _ => unreachable!(),
             }
