@@ -111,7 +111,25 @@ pub fn build() -> App<'static, 'static> {
                          .short("s")
                          .multiple(true)
                          .help("Encrypt with a password \
-                                (can be given multiple times)")))
+                                (can be given multiple times)"))
+                    .arg(Arg::with_name("mode").value_name("MODE")
+                         .long("mode")
+                         .possible_values(&["transport", "rest", "all"])
+                         .default_value("all")
+                         .help("Selects what kind of keys are considered for \
+                                encryption.  Transport select subkeys marked \
+                                as suitable for transport encryption, rest \
+                                selects those for encrypting data at rest, \
+                                and all selects all encryption-capable \
+                                subkeys"))
+                    .arg(Arg::with_name("compression")
+                         .value_name("KIND")
+                         .long("compression")
+                         .possible_values(&["none", "pad", "zip", "zlib",
+                                            "bzip2"])
+                         .default_value("pad")
+                         .help("Selects compression scheme to use")))
+
         .subcommand(SubCommand::with_name("sign")
                     .display_order(25)
                     .about("Signs a message")
@@ -350,7 +368,7 @@ pub fn build() -> App<'static, 'static> {
                              .long("cipher-suite")
                              .short("c")
                              .possible_values(&["rsa3k", "cv25519"])
-                             .default_value("rsa3k")
+                             .default_value("cv25519")
                              .help("Cryptographic algorithms used for the key."))
                         .arg(Arg::with_name("with-password")
                              .long("with-password")
@@ -380,12 +398,12 @@ pub fn build() -> App<'static, 'static> {
                         .arg(Arg::with_name("can-encrypt").value_name("PURPOSE")
                              .long("can-encrypt")
                              .possible_values(&["transport", "rest", "all"])
-                             .default_value("all")
                              .help("The key has an encryption-capable subkey \
                                     (default)"))
                         .arg(Arg::with_name("cannot-encrypt")
                              .long("cannot-encrypt")
                              .help("The key will not be able to encrypt data"))
+
                         .arg(Arg::with_name("export").value_name("OUTFILE")
                              .long("export")
                              .short("e")
@@ -432,7 +450,31 @@ pub fn build() -> App<'static, 'static> {
                                      .short("p")
                                      .help("Sets the prefix to use for output files \
                                             (defaults to the input filename with a dash, \
-                                            or 'output')"))))
+                                            or 'output')")))
+                    .subcommand(SubCommand::with_name("join")
+                                .about("Joins OpenPGP packets split across \
+                                        files")
+                                .arg(Arg::with_name("input").value_name("FILE")
+                                     .multiple(true)
+                                     .help("Sets the input files to use"))
+                                .arg(Arg::with_name("output").value_name("FILE")
+                                     .long("output")
+                                     .short("o")
+                                     .help("Sets the output file to use"))
+                                .arg(Arg::with_name("kind")
+                                     .value_name("KIND")
+                                     .long("kind")
+                                     .possible_values(&["message", "publickey",
+                                                        "secretkey",
+                                                        "signature", "file"])
+                                     .default_value("file")
+                                     .help("Selects the kind of header line to \
+                                            produce"))
+                                .arg(Arg::with_name("binary")
+                                     .long("binary")
+                                     .short("B")
+                                     .help("Don't ASCII-armor encode the \
+                                            OpenPGP data"))))
 
         .subcommand(SubCommand::with_name("wkd")
                     .about("Interacts with Web Key Directories")
@@ -447,7 +489,7 @@ pub fn build() -> App<'static, 'static> {
                                             obtain the WKD URI.")))
                     .subcommand(SubCommand::with_name("get")
                                 .about("Writes to the standard output the \
-                                        Transferable Public Key retrieved \
+                                        TPK retrieved \
                                         from a Web Key Directory, given an \
                                         email address")
                                 .arg(Arg::with_name("input")
